<?php

namespace Drupal\recurly_aegir\Processors;

use Drupal\Component\Utility\Html;
use Drupal\Core\Url;
use Drupal\Core\Link;

/**
 * Processes Recurly notifications for Aegir.
 */
abstract class RecurlyAegirProcessor {

  /**
   * @var SimpleXMLElement|NULL $account
   *   Account information.
   */
  protected $account;

  /**
   * @var SimpleXMLElement|NULL $subscription
   *   Subscription information.
   */
  protected $subscription;

  /**
   * @var SimpleXMLElement|NULL $transaction
   *   Transaction information.
   */
  protected $transaction;

  /**
   * @var SimpleXMLElement|NULL $invoice
   *   Invoice information.
   */
  protected $invoice;

  /**
   * @var boolean $result
   *   TRUE if processing was successful; FALSE otherwise.
   */
  protected $result = FALSE;

  /**
   * Class constuctor.
   *
   * @param SimpleXMLElement|NULL $account
   *   Account information.
   * @param SimpleXMLElement|NULL $subscription
   *   Subscription information.
   * @param SimpleXMLElement|NULL $transaction
   *   Transaction information.
   * @param SimpleXMLElement|NULL $invoice
   *   Invoice information.
   * @return $this
   *   The newly created object.
   */
  public function __construct(
      $account = NULL,
      $subscription = NULL,
      $transaction = NULL,
      $invoice = NULL) {
    $this->account = $account;
    $this->subscription = $subscription;
    $this->transaction = $transaction;
    $this->invoice = $invoice;
  }

  /**
   * Process any notifications coming from Recurly.
   *
   * Must be overriden by subclasses to actually do something.
   *
   * @return $this
   *   The object itself.
   */
  abstract public function process();

  /**
   * Get the processing result.
   *
   * @return boolean
   *   TRUE if processing was successful; FALSE otherwise.
   */
  public function getResult() {
    return $this->result;
  }

  /**
   * Gets a user ID from an account code.
   *
   * Given a Recurly account code, returns a local Drupal user ID.
   *
   * @param SimpleXMLElement $account
   *   The user's account code within Recurly.
   * @return integer
   *   The local Drupal user ID.
   *
   * @todo Add support for account entities other than User.
   * @todo Move this into the base Recurly module.
   */
  protected function getUserId(\SimpleXMLElement $account) {
    $delimiter = '-';

    // Throw away the first part of the code as we only care about the ID.
    strtok((string) $account->account_code, $delimiter);

    // Grab the second part as the user ID.
    $user_id = strtok($delimiter);

    // Compare the internal username with the one provided by Recurly.
    $user = \Drupal\user\Entity\User::load($user_id);
    if ($user->get('name')->value != (string) $account->username) {
      $message = "User ID %userid: Drupal username %drupal does not match Recurly username %recurly.";
      \Drupal::logger('recurly_aegir')->warning($message, [
        '%userid' => $user_id,
        '%drupal' => $user->get('name')->value,
        '%recurly' => (string) $account->username,
      ]);
    }

    // Compare the internal e-mail address with the one provided by Recurly.
    if ($user->get('mail')->value != (string) $account->email) {
      $message = "User ID %userid: Drupal e-mail %drupal does not match Recurly e-mail %recurly.";
      \Drupal::logger('recurly_aegir')->warning($message, [
        '%userid' => $user_id,
        '%drupal' => $user->get('mail')->value,
        '%recurly' => (string) $account->email,
      ]);
    }

    // Return it.
    return $user_id;
  }

  /**
   * Gets the Recurly subscription URL from a UUID.
   *
   * @param string $uuid
   * @return string
   *   The subscription URL on the Recurly subdomain.
   */
  public static function getSubscriptionUrl($uuid) {
    $subdomain = \Drupal::config('recurly.settings')->get('recurly_subdomain');
    return 'https://' . $subdomain . '.recurly.com/subscriptions/' . $uuid;
  }

  /**
   * Gets a subscription UUID from a URL.
   *
   * @param string $url
   *   The subscription's URL on the Recurly site.
   * @return string
   *   The subscription's UUID.
   */
  public static function getSubscriptionUuid($url) {
    $pieces = explode('/', $url);
    return $pieces[4];
  }

  /**
   * Gets the configuration link for a specific site.
   *
   * @param Node $site
   *   The site node.
   * @return Link
   *   The site node's configuration link.
   */
  public static function getSiteConfigurationLink($site) {
    $url = Url::fromRoute('entity.node.edit_form', ['node' => $site->id()]);
    return Link::fromTextAndUrl(t('<em>Configuration required!</em>'), $url);
  }

  /**
   * Gets the link for a specific client site.
   *
   * @param Node $site
   *   The site node.
   * @return Link
   *   A link to the site itself.
   */
  public static function getSiteLink($site) {
    $protocol = \Drupal::request()->getScheme();
    $subdomain = Html::escape($site->getTitle());
    $domain = \Drupal::request()->getHost();
    $hostname = "$subdomain.$domain";
    $url = Url::fromUri("$protocol://$hostname/");
    return Link::fromTextAndUrl($hostname, $url);
  }
}
